/*-------------------------------------------------------------------------
 *
 * pg_ts_template_d.h
 *    Macro definitions for pg_ts_template
 *
 * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_TS_TEMPLATE_D_H
#define PG_TS_TEMPLATE_D_H

/* Macros related to the structure of pg_ts_template */

#define TSTemplateRelationId 3764
#define TSTemplateNameNspIndexId 3766
#define TSTemplateOidIndexId 3767

#define Anum_pg_ts_template_oid 1
#define Anum_pg_ts_template_tmplname 2
#define Anum_pg_ts_template_tmplnamespace 3
#define Anum_pg_ts_template_tmplinit 4
#define Anum_pg_ts_template_tmpllexize 5

#define Natts_pg_ts_template 5

/* Definitions copied from pg_ts_template.h */


/* OID symbols for objects defined in pg_ts_template.dat */


#endif							/* PG_TS_TEMPLATE_D_H */
