/*-------------------------------------------------------------------------
 *
 * pg_amproc_d.h
 *    Macro definitions for pg_amproc
 *
 * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_AMPROC_D_H
#define PG_AMPROC_D_H

/* Macros related to the structure of pg_amproc */

#define AccessMethodProcedureRelationId 2603
#define AccessMethodProcedureIndexId 2655
#define AccessMethodProcedureOidIndexId 2757

#define Anum_pg_amproc_oid 1
#define Anum_pg_amproc_amprocfamily 2
#define Anum_pg_amproc_amproclefttype 3
#define Anum_pg_amproc_amprocrighttype 4
#define Anum_pg_amproc_amprocnum 5
#define Anum_pg_amproc_amproc 6

#define Natts_pg_amproc 6

/* Definitions copied from pg_amproc.h */


/* OID symbols for objects defined in pg_amproc.dat */


#endif							/* PG_AMPROC_D_H */
