/*-------------------------------------------------------------------------
 *
 * pg_inherits_d.h
 *    Macro definitions for pg_inherits
 *
 * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_INHERITS_D_H
#define PG_INHERITS_D_H

/* Macros related to the structure of pg_inherits */

#define InheritsRelationId 2611
#define InheritsRelidSeqnoIndexId 2680
#define InheritsParentIndexId 2187

#define Anum_pg_inherits_inhrelid 1
#define Anum_pg_inherits_inhparent 2
#define Anum_pg_inherits_inhseqno 3
#define Anum_pg_inherits_inhdetachpending 4

#define Natts_pg_inherits 4

/* Definitions copied from pg_inherits.h */


/* OID symbols for objects defined in pg_inherits.dat */


#endif							/* PG_INHERITS_D_H */
