/* ----------
 *	pg_trace.h
 *
 *	Definitions for the PostgreSQL tracing framework
 *
 *	Copyright (c) 2006-2025, PostgreSQL Global Development Group
 *
 *	src/include/pg_trace.h
 * ----------
 */

#ifndef PG_TRACE_H
#define PG_TRACE_H

#include "utils/probes.h"		/* IWYU pragma: export */

#endif							/* PG_TRACE_H */
