/*-------------------------------------------------------------------------
 *
 * pg_user_mapping_d.h
 *    Macro definitions for pg_user_mapping
 *
 * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_USER_MAPPING_D_H
#define PG_USER_MAPPING_D_H

/* Macros related to the structure of pg_user_mapping */

#define UserMappingRelationId 1418
#define UserMappingOidIndexId 174
#define UserMappingUserServerIndexId 175

#define Anum_pg_user_mapping_oid 1
#define Anum_pg_user_mapping_umuser 2
#define Anum_pg_user_mapping_umserver 3
#define Anum_pg_user_mapping_umoptions 4

#define Natts_pg_user_mapping 4

/* Definitions copied from pg_user_mapping.h */


/* OID symbols for objects defined in pg_user_mapping.dat */


#endif							/* PG_USER_MAPPING_D_H */
